@@ -5,7 +5,8 @@
 //C- Copyright (c) 2001  AT&T
 //C-
 //C- This software is subject to, and may be distributed under, the
-//C- GNU General Public License, Version 2. The license should have
+//C- GNU General Public License, either Version 2 of the license,
+//C- or (at your option) any later version. The license should have
 //C- accompanied the software or you may obtain a copy of the license
 //C- from the Free Software Foundation at http://www.fsf.org .
 //C-
@@ -14,10 +15,10 @@
 //C- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 //C- GNU General Public License for more details.
 //C- 
-//C- DjVuLibre-3.5 is derived from the DjVu(r) Reference Library
-//C- distributed by Lizardtech Software.  On July 19th 2002, Lizardtech 
-//C- Software authorized us to replace the original DjVu(r) Reference 
-//C- Library notice by the following text (see doc/lizard2002.djvu):
+//C- DjVuLibre-3.5 is derived from the DjVu(r) Reference Library from
+//C- Lizardtech Software.  Lizardtech Software has authorized us to
+//C- replace the original DjVu(r) Reference Library notice by the following
+//C- text (see doc/lizard2002.djvu and doc/lizardtech2007.djvu):
 //C-
 //C-  ------------------------------------------------------------------
 //C- | DjVu (r) Reference Library (v. 3.5)
@@ -26,7 +27,8 @@
 //C- | 6,058,214 and patents pending.
 //C- |
 //C- | This software is subject to, and may be distributed under, the
-//C- | GNU General Public License, Version 2. The license should have
+//C- | GNU General Public License, either Version 2 of the license,
+//C- | or (at your option) any later version. The license should have
 //C- | accompanied the software or you may obtain a copy of the license
 //C- | from the Free Software Foundation at http://www.fsf.org .
 //C- |
@@ -117,7 +119,10 @@
 #endif
 
 #define XP_UNIX 1
+#define MOZ_X11 1
 #include "npapi.h"
+#include "npruntime.h"
+#include "npupp.h"
 
 #include <X11/Intrinsic.h>
 #include <X11/IntrinsicP.h>
@@ -130,7 +135,12 @@
 #ifndef FALSE
 # define FALSE 0
 #endif
-
+#ifndef offsetof
+# define offsetof(TYPE,MEMBER) ((size_t)&((TYPE*)0)->MEMBER)
+#endif
+#ifndef min
+# define min(a,b) (((a)<(b))?(a):(b))
+#endif
 
 
 /* ------------------------------------------------------------ */
@@ -2227,7 +2237,7 @@ NPP_GetJavaClass(void)
 
 
 NPError
-NPP_GetValue(void *future, NPPVariable variable, void *value)
+NPP_GetValue(NPP instance, NPPVariable variable, void *value)
 {
    NPError err = NPERR_NO_ERROR;
    switch (variable)
@@ -2261,8 +2271,297 @@ NPP_GetValue(void *future, NPPVariable variable, void *value)
 
 
 /******************************************************************************
-*************** The Netscape SDK gets included here ***************************
+*************** This replaces npunix.c from the Netscape SDK ******************
+******** Derived from nspluginwrapper licensed under GPL v2 or later **********
 *******************************************************************************/
 
-#include "npunix.c"
+
+char *
+NP_GetMIMEDescription(void)
+{
+  return NPP_GetMIMEDescription();
+}
+
+NPError
+NP_GetValue(void *future, NPPVariable variable, void *value)
+{
+  return NPP_GetValue((NPP)future, variable, value);
+}
+
+NPError
+NP_Shutdown(void)
+{
+  NPP_Shutdown();
+  return NPERR_NO_ERROR;
+}
+
+static NPNetscapeFuncs mozilla_funcs;
+
+static int mozilla_has_npruntime = 0;
+
+void
+NPN_Version(int* plugin_major, int* plugin_minor,
+            int* mozilla_major, int* mozilla_minor)
+{
+  *plugin_major = NP_VERSION_MAJOR;
+  *plugin_minor = NP_VERSION_MINOR;
+  *mozilla_major = mozilla_funcs.version >> 8;
+  *mozilla_minor = mozilla_funcs.version & 0xFF;
+}
+
+NPError
+NPN_GetValue(NPP instance, NPNVariable variable, void *r_value)
+{
+  if (mozilla_funcs.getvalue == NULL)
+    return NPERR_INVALID_FUNCTABLE_ERROR;
+  return CallNPN_GetValueProc(mozilla_funcs.getvalue,
+                              instance, variable, r_value);
+}
+
+NPError
+NPN_GetURL(NPP instance, const char* url, const char* window)
+{
+  if (mozilla_funcs.geturl == NULL)
+    return NPERR_INVALID_FUNCTABLE_ERROR;
+  return CallNPN_GetURLProc(mozilla_funcs.geturl, instance, url, window);
+}
+
+NPError
+NPN_GetURLNotify(NPP instance, const char* url, const char* window, void * data)
+{
+  if (mozilla_funcs.geturlnotify == NULL)
+    return NPERR_INVALID_FUNCTABLE_ERROR;
+  return CallNPN_GetURLNotifyProc(mozilla_funcs.geturlnotify, 
+                                  instance, url, window, data);
+}
+
+NPError
+NPN_PostURL(NPP instance, const char* url, const char* window,
+            uint32 len, const char* buf, NPBool file)
+{
+  if (mozilla_funcs.posturl == NULL)
+    return NPERR_INVALID_FUNCTABLE_ERROR;
+  return CallNPN_PostURLProc(mozilla_funcs.posturl, instance,
+                             url, window, len, buf, file);
+}
+
+NPError
+NPN_RequestRead(NPStream* stream, NPByteRange* rangeList)
+{
+  if (mozilla_funcs.requestread == NULL)
+    return NPERR_INVALID_FUNCTABLE_ERROR;
+  return CallNPN_RequestReadProc(mozilla_funcs.requestread,
+                                 stream, rangeList);
+}
+
+NPError
+NPN_NewStream(NPP instance, NPMIMEType type, const char *window,
+	      NPStream** stream_ptr)
+{
+  if (mozilla_funcs.newstream == NULL)
+    return NPERR_INVALID_FUNCTABLE_ERROR;
+  return CallNPN_NewStreamProc(mozilla_funcs.newstream, instance,
+                               type, window, stream_ptr);
+}
+
+int32
+NPN_Write(NPP instance, NPStream* stream, int32 len, void* buffer)
+{
+  if (mozilla_funcs.write == NULL)
+    return -1;
+  return CallNPN_WriteProc(mozilla_funcs.write, instance,
+                           stream, len, buffer);
+}
+
+NPError
+NPN_DestroyStream(NPP instance, NPStream* stream, NPError reason)
+{
+  if (mozilla_funcs.destroystream == NULL)
+    return NPERR_INVALID_FUNCTABLE_ERROR;
+  return CallNPN_DestroyStreamProc(mozilla_funcs.destroystream,
+                                   instance, stream, reason);
+}
+
+void
+NPN_Status(NPP instance, const char* message)
+{
+  CallNPN_StatusProc(mozilla_funcs.status, instance, message);
+}
+
+const char*
+NPN_UserAgent(NPP instance)
+{
+  return CallNPN_UserAgentProc(mozilla_funcs.uagent, instance);
+}
+
+void*
+NPN_MemAlloc(uint32 size)
+{
+  return CallNPN_MemAllocProc(mozilla_funcs.memalloc, size);
+}
+
+void 
+NPN_MemFree(void* ptr)
+{
+  CallNPN_MemFreeProc(mozilla_funcs.memfree, ptr);
+}
+
+void 
+NPN_ReloadPlugins(NPBool reloadPages)
+{
+  if (mozilla_funcs.reloadplugins != NULL)
+    CallNPN_ReloadPluginsProc(mozilla_funcs.reloadplugins, reloadPages);
+}
+
+NPError
+NPN_SetValue(NPP instance, NPPVariable variable, void *value)
+{
+  if (mozilla_funcs.setvalue == NULL)
+    return NPERR_INVALID_FUNCTABLE_ERROR;
+  return CallNPN_GetValueProc(mozilla_funcs.setvalue, 
+                              instance, variable, value);
+}
+
+void
+NPN_InvalidateRect(NPP instance, NPRect *invalid)
+{
+  if (mozilla_funcs.invalidaterect != NULL)
+    CallNPN_InvalidateRectProc(mozilla_funcs.invalidaterect, instance, invalid);
+}
+
+void
+NPN_InvalidateRegion(NPP instance, NPRegion invalid)
+{
+  if (mozilla_funcs.invalidateregion != NULL)
+    CallNPN_InvalidateRegionProc(mozilla_funcs.invalidateregion, instance, invalid);
+}
+
+void
+NPN_ForceRedraw(NPP instance)
+{
+  if (mozilla_funcs.forceredraw != NULL)
+    CallNPN_ForceRedrawProc(mozilla_funcs.forceredraw, instance);
+}
+
+NPIdentifier 
+NPN_GetStringIdentifier(const NPUTF8 *name)
+{
+  if (!mozilla_funcs.getstringidentifier || !mozilla_has_npruntime)
+    return NULL;
+  return CallNPN_GetStringIdentifierProc(mozilla_funcs.getstringidentifier, name);
+}
+
+NPIdentifier 
+NPN_GetIntIdentifier(int32_t intid)
+{
+  if (!mozilla_funcs.getintidentifier || !mozilla_has_npruntime)
+    return NULL;
+  return CallNPN_GetIntIdentifierProc(mozilla_funcs.getintidentifier, intid);
+}
+
+bool 
+NPN_IdentifierIsString(NPIdentifier identifier)
+{
+  if (!mozilla_funcs.identifierisstring || !mozilla_has_npruntime)
+    return FALSE;
+  return CallNPN_IdentifierIsStringProc(mozilla_funcs.identifierisstring, 
+                                        identifier);
+}
+
+NPUTF8 *
+NPN_UTF8FromIdentifier(NPIdentifier identifier)
+{
+  if (!mozilla_funcs.utf8fromidentifier || !mozilla_has_npruntime)
+    return NULL;
+  return CallNPN_UTF8FromIdentifierProc(mozilla_funcs.utf8fromidentifier, 
+                                                identifier);
+}
+
+int32_t 
+NPN_IntFromIdentifier(NPIdentifier identifier)
+{
+  if (!mozilla_funcs.intfromidentifier || !mozilla_has_npruntime)
+    return 0;
+  return CallNPN_IntFromIdentifierProc(mozilla_funcs.intfromidentifier, 
+                                       identifier);
+}
+
+NPObject *
+NPN_CreateObject(NPP npp, NPClass *aClass)
+{
+  if (!mozilla_funcs.createobject || !mozilla_has_npruntime)
+    return 0;
+  return CallNPN_CreateObjectProc(mozilla_funcs.createobject, npp, aClass); 
+}
+
+NPObject *
+NPN_RetainObject(NPObject *npobj)
+{
+  if (!mozilla_funcs.retainobject || !mozilla_has_npruntime)
+    return npobj;
+  return CallNPN_RetainObjectProc(mozilla_funcs.retainobject, npobj);
+}
+
+void 
+NPN_ReleaseObject(NPObject *npobj)
+{
+  if (mozilla_funcs.releaseobject && mozilla_has_npruntime)
+    return CallNPN_ReleaseObjectProc(mozilla_funcs.releaseobject, npobj);
+}
+
+void 
+NPN_SetException(NPObject *npobj, const NPUTF8 *message)
+{
+  if (mozilla_funcs.setexception && mozilla_has_npruntime)
+    CallNPN_SetExceptionProc(mozilla_funcs.setexception, npobj, message);
+}
+
+NPError
+NP_Initialize(NPNetscapeFuncs *moz_funcs, NPPluginFuncs *plugin_funcs)
+{
+  /* basic checks */
+  if (moz_funcs == NULL || plugin_funcs == NULL)
+    return NPERR_INVALID_FUNCTABLE_ERROR;
+  if ((moz_funcs->version >> 8) != NP_VERSION_MAJOR)
+    return NPERR_INCOMPATIBLE_VERSION_ERROR;
+  if (moz_funcs->size < (offsetof(NPNetscapeFuncs, forceredraw) 
+                         + sizeof(NPN_ForceRedrawUPP) ))
+    return NPERR_INVALID_FUNCTABLE_ERROR;
+  if (plugin_funcs->size < sizeof(NPPluginFuncs))
+    return NPERR_INVALID_FUNCTABLE_ERROR;
+  
+  /* copy mozilla_funcs. */
+  memcpy(&mozilla_funcs, moz_funcs, min(moz_funcs->size, sizeof(mozilla_funcs)));
+
+  /* fill plugin_funcs. */
+  memset(plugin_funcs, 0, sizeof(*plugin_funcs));
+  plugin_funcs->size = sizeof(NPPluginFuncs);
+  plugin_funcs->version = (NP_VERSION_MAJOR << 8) + NP_VERSION_MINOR;
+  plugin_funcs->newp = NewNPP_NewProc(NPP_New);
+  plugin_funcs->destroy = NewNPP_DestroyProc(NPP_Destroy);
+  plugin_funcs->setwindow = NewNPP_SetWindowProc(NPP_SetWindow);
+  plugin_funcs->newstream = NewNPP_NewStreamProc(NPP_NewStream);
+  plugin_funcs->destroystream = NewNPP_DestroyStreamProc(NPP_DestroyStream);
+  plugin_funcs->asfile = NewNPP_StreamAsFileProc(NPP_StreamAsFile);
+  plugin_funcs->writeready = NewNPP_WriteReadyProc(NPP_WriteReady);
+  plugin_funcs->write = NewNPP_WriteProc(NPP_Write);
+  plugin_funcs->print = NewNPP_PrintProc(NPP_Print);
+  plugin_funcs->event = NULL;
+  plugin_funcs->urlnotify = NewNPP_URLNotifyProc(NPP_URLNotify);
+  plugin_funcs->javaClass = NULL;
+  plugin_funcs->getvalue = NewNPP_GetValueProc(NPP_GetValue);
+  plugin_funcs->setvalue = NULL;
+
+  /* check for npruntime */
+  mozilla_has_npruntime = 1;
+  if ((moz_funcs->version >> 8) == 0 && (moz_funcs->version & 0xff) < 14)
+    mozilla_has_npruntime = 0;
+  if (moz_funcs->size < (offsetof(NPNetscapeFuncs, setexception) 
+                         + sizeof(NPN_SetExceptionUPP) ))
+    mozilla_has_npruntime = 0;
+  
+  /* call npp_initialize */
+  return NPP_Initialize();
+}
+
 
