@@ -15,7 +15,7 @@
 //C- GNU General Public License for more details.
 //C-  ------------------------------------------------------------------
 
-// $Id: djview.cpp,v 1.54 2007/12/31 20:09:40 leonb Exp $
+// $Id: djview.cpp,v 1.62 2008/08/04 17:06:27 bpearlmutter Exp $
 
 #if AUTOCONF
 # include "config.h"
@@ -44,6 +44,7 @@
 #include <QLocale>
 #include <QRegExp>
 #include <QSettings>
+#include <QSessionManager>
 #include <QString>
 #include <QStringList>
 #include <QTranslator>
@@ -149,11 +150,6 @@ addDirectory(QStringList &dirs, QString path)
 QDjViewApplication::QDjViewApplication(int &argc, char **argv)
   : QApplication(argc, argv), context(argv[0])
 {
-  // Application data
-  setOrganizationName(DJVIEW_ORG);
-  setOrganizationDomain(DJVIEW_DOMAIN);
-  setApplicationName(DJVIEW_APP);
-  
   // Translators
   QTranslator *qtTrans = new QTranslator(this);
   QTranslator *djviewTrans = new QTranslator(this);
@@ -165,12 +161,12 @@ QDjViewApplication::QDjViewApplication(int &argc, char **argv)
 #ifdef LC_MESSAGES
   QString varLcMessages = ::setlocale(LC_MESSAGES, 0);
   if (varLcMessages.size())
-    langs += varLcMessages;
+    langs += varLcMessages.toLower();
 #else
 # ifdef LC_ALL
   QString varLcMessages = ::setlocale(LC_ALL, 0);
   if (varLcMessages.size())
-    langs += varLcMessages;
+    langs += varLcMessages.toLower();
 # endif
 #endif
 #ifdef Q_WS_MAC
@@ -219,7 +215,7 @@ QDjViewApplication::QDjViewApplication(int &argc, char **argv)
           if (! djviewTransValid && qdir.exists())
             djviewTransValid= djviewTrans->load("djview_" + lang, dir, "_.-");
         }
-      if (lang == "en") 
+      if (lang == "en" || lang.startsWith("en_") || lang == "c")
         break;
     }
   // - install tranlators
@@ -268,6 +264,51 @@ QDjViewApplication::event(QEvent *ev)
 }
 
 
+#ifdef Q_WS_X11
+
+void 
+QDjViewApplication::commitData(QSessionManager &)
+{
+}
+ 
+
+void 
+QDjViewApplication::saveState(QSessionManager &sm)
+{
+  int n = 0;
+  QSettings s;
+  foreach(QWidget *w, topLevelWidgets())
+    {
+      QDjView *djview = qobject_cast<QDjView*>(w);
+      if (djview && !djview->objectName().isEmpty() &&
+          djview->getViewerMode() == QDjView::STANDALONE )
+        {
+          if (++n == 1)
+            {
+              QStringList discard = QStringList(applicationFilePath());
+              discard << QLatin1String("-discard") << sessionId();
+              sm.setDiscardCommand(discard);
+              QString id = QLatin1String("session_") + sessionId();
+              s.sync();
+              s.remove(id);
+              s.beginGroup(id);
+            }
+          s.beginGroup(QString::number(n));
+          djview->saveSession(&s);
+          s.endGroup();
+        }
+    }
+  if (n > 0)
+    {
+      s.setValue("windows", n);
+      s.endGroup();
+      s.sync();
+    }
+}
+
+#endif
+
+
 int 
 main(int argc, char *argv[])
 {
@@ -277,7 +318,12 @@ main(int argc, char *argv[])
   ::setlocale(LC_NUMERIC, "C");
 #endif
 
-  // Message verbosity
+  // Application data
+  QApplication::setOrganizationName(DJVIEW_ORG);
+  QApplication::setOrganizationDomain(DJVIEW_DOMAIN);
+  QApplication::setApplicationName(DJVIEW_APP);
+  
+  // Message handler
   qtDefaultHandler = qInstallMsgHandler(qtMessageHandler);
 #ifdef Q_OS_UNIX
   const char *s = ::getenv("DJVIEW_VERBOSE");
@@ -293,20 +339,53 @@ main(int argc, char *argv[])
   
   // Plugin mode
 #ifdef Q_WS_X11
-  for (int i=1; i<argc; i++)
-    if (!strcmp(argv[i],"-netscape") || !strcmp(argv[i],"--netscape"))
-      { // run as plugin
-        verbose = true;
-        QDjViewPlugin dispatcher(argv[0]);
-        return dispatcher.exec();
-      }
+  if (argc==2 && !strcmp(argv[1],"-netscape"))
+    {
+      verbose = true;
+      QDjViewPlugin dispatcher(argv[0]);
+      return dispatcher.exec();
+    }
 #endif
-
-  // Start
+  
+  // Discard session
+#ifdef Q_WS_X11
+  if (argc==3 && !strcmp(argv[1],"-discard"))
+    {
+      QSettings s;
+      s.remove(QLatin1String("session_")+QLatin1String(argv[2]));
+      s.sync();
+      return 0;
+    }
+#endif
+  
+  // Create application
   QDjViewApplication app(argc, argv);
-  QDjView *main = app.newWindow();
+
+  // Restore session
+#ifdef Q_WS_X11
+  if (app.isSessionRestored())
+    {
+      QSettings s;
+      s.beginGroup(QLatin1String("session_") + app.sessionId());
+      int windows = s.value("windows").toInt();
+      if (windows > 0)
+        {
+          for (int n=1; n<=windows; n++)
+            {
+              QDjView *w = new QDjView(*app.djvuContext());
+              w->setAttribute(Qt::WA_DeleteOnClose);
+              s.beginGroup(QString::number(n));
+              w->restoreSession(&s);
+              s.endGroup();
+              w->show();
+            }
+          return app.exec();
+        }
+    }
+#endif
   
   // Process command line
+  QDjView *main = app.newWindow();
   while (argc > 1 && argv[1][0] == '-')
     {
       QString arg = QString::fromLocal8Bit(argv[1]).replace(QRegExp("^-+"),"");
@@ -342,7 +421,7 @@ main(int argc, char *argv[])
         }
     }
 
-  // process events
+  // Process events
   main->show();
   return app.exec();
 }
