@@ -15,7 +15,7 @@
 //C- GNU General Public License for more details.
 //C-  ------------------------------------------------------------------
 
-// $Id: djview.h,v 1.2 2007/11/04 03:20:34 leonb Exp $
+// $Id: djview.h,v 1.4 2008/02/07 05:16:45 leonb Exp $
 
 #ifndef DJVIEW_H
 #define DJVIEW_H
@@ -27,8 +27,9 @@
 #include <Qt>
 #include <QObject>
 #include <QApplication>
-#include <QPointer>
 #include <QEvent>
+#include <QPointer>
+#include <QString>
 
 #include "qdjvu.h"
 
@@ -45,7 +46,18 @@ class QDjViewApplication : public QApplication
   QDjView *newWindow();
  protected:
   bool event(QEvent *ev);
+#ifdef Q_WS_X11
+  void commitData(QSessionManager &sm);
+  void saveState(QSessionManager &sm);
+#endif
 };
 
 
 #endif
+
+
+/* -------------------------------------------------------------
+   Local Variables:
+   c++-font-lock-extra-types: ( "\\sw+_t" "[A-Z]\\sw*[a-z]\\sw*" )
+   End:
+   ------------------------------------------------------------- */
