@@ -15,18 +15,18 @@
 //C- GNU General Public License for more details.
 //C-  ------------------------------------------------------------------
 
-// $Id: qdjviewexporters.cpp,v 1.23 2007/12/31 20:09:40 leonb Exp $
+// $Id: qdjviewexporters.cpp,v 1.26 2008/08/05 06:03:37 leonb Exp $
 
 #if AUTOCONF
 # include "config.h"
 #else
 # define HAVE_STRING_H     1
 # define HAVE_SYS_TYPES_H  1
-# define HAVE_UNISTD_H     1
 # define HAVE_STRERROR     1
 # ifndef WIN32
 #  define HAVE_SYS_WAIT_H  1
 #  define HAVE_WAITPID     1
+#  define HAVE_UNISTD_H     1
 # endif
 #endif
 
@@ -49,9 +49,14 @@
 # include <unistd.h>
 #endif
 #if HAVE_TIFF
+# include <tiff.h>
 # include <tiffio.h>
+# include <tiffconf.h>
 # include "tiff2pdf.h"
 #endif
+#ifdef WIN32
+# include <io.h>
+#endif
 
 #include <QApplication>
 #include <QCheckBox>
@@ -978,9 +983,15 @@ QDjViewPSExporter::printSetup(QPrintDialog *dialog, bool dir)
       copies = 1;
       collate = true;
       lastfirst = false;
+#if QT_VERSION >= 0x40400
+      QSpinBox* lcop = qFindChild<QSpinBox*>(dialog, "copies");
+      QCheckBox* lcol = qFindChild<QCheckBox*>(dialog, "collate");
+      QCheckBox* lplf = qFindChild<QCheckBox*>(dialog, "reverse");
+#else
       QSpinBox* lcop = qFindChild<QSpinBox*>(dialog, "sbNumCopies");
       QCheckBox* lcol = qFindChild<QCheckBox*>(dialog, "chbCollate");
       QCheckBox* lplf = qFindChild<QCheckBox*>(dialog, "chbPrintLastFirst");
+#endif
       if (lcop)
         copies = qMax(1, lcop->value());
       if (lcol)
