@@ -15,7 +15,7 @@
 //C- GNU General Public License for more details.
 //C-  ------------------------------------------------------------------
 
-// $Id: qdjviewplugin.cpp,v 1.42 2007/12/31 20:09:40 leonb Exp $
+// $Id: qdjviewplugin.cpp,v 1.43 2008/07/31 15:24:42 leonb Exp $
 
 #if AUTOCONF
 # include "config.h"
@@ -532,8 +532,6 @@ QDjViewPlugin::Instance::restore(QDjVuWidget *widget)
       QDjVuWidget::DisplayMode mode 
         = (QDjVuWidget::DisplayMode)((q[0] & 0xf0000) >> 16);
       QDjVuWidget::Position pos;
-      pos.anchorRight = false;
-      pos.anchorBottom = false;
       pos.inPage = false;
       pos.pageNo = (q[1] & 0xfffffff);
       pos.posView.rx() = q[2];
