@@ -15,7 +15,7 @@
 //C- GNU General Public License for more details.
 //C-  ------------------------------------------------------------------
 
-// $Id: qdjviewprefs.h,v 1.35 2008/01/21 14:40:00 leonb Exp $
+// $Id: qdjviewprefs.h,v 1.40 2008/08/03 15:20:16 leonb Exp $
 
 #ifndef QDJVIEWPREFS_H
 #define QDJVIEWPREFS_H
@@ -50,7 +50,7 @@
 # define DJVIEW_APP "DjView"
 #endif
 #ifndef DJVIEW_VERSION
-# define DJVIEW_VERSION 0x40300
+# define DJVIEW_VERSION 0x40400
 #endif
 
 
@@ -67,21 +67,23 @@ class QDjViewPrefs : public QObject
 public:
 
   enum Option {
-    SHOW_MENUBAR        = 0x000001,
-    SHOW_TOOLBAR        = 0x000002,
-    SHOW_SIDEBAR        = 0x000004,
-    SHOW_STATUSBAR      = 0x000010,
-    SHOW_SCROLLBARS     = 0x000020,
-    SHOW_FRAME          = 0x000100,
-    SHOW_MAPAREAS       = 0x000200,
-    SHOW_MASK           = 0x000FFF, 
-    LAYOUT_CONTINUOUS   = 0x001000,
-    LAYOUT_SIDEBYSIDE   = 0x002000,
-    LAYOUT_MASK         = 0x00F000,
-    HANDLE_MOUSE        = 0x100000,
-    HANDLE_KEYBOARD     = 0x200000,
-    HANDLE_LINKS        = 0x400000,
-    HANDLE_CONTEXTMENU  = 0x800000,
+    SHOW_MENUBAR          = 0x000001,
+    SHOW_TOOLBAR          = 0x000002,
+    SHOW_SIDEBAR          = 0x000004,
+    SHOW_STATUSBAR        = 0x000010,
+    SHOW_SCROLLBARS       = 0x000020,
+    SHOW_FRAME            = 0x000100,
+    SHOW_MAPAREAS         = 0x000200,
+    SHOW_MASK             = 0x000FFF, 
+    LAYOUT_CONTINUOUS     = 0x001000,
+    LAYOUT_SIDEBYSIDE     = 0x002000,
+    LAYOUT_COVERPAGE      = 0x004000,
+    LAYOUT_RIGHTTOLEFT    = 0x008000,
+    LAYOUT_MASK           = 0x00F000,
+    HANDLE_MOUSE          = 0x100000,
+    HANDLE_KEYBOARD       = 0x200000,
+    HANDLE_LINKS          = 0x400000,
+    HANDLE_CONTEXTMENU    = 0x800000,
   };
 
   enum Tool {
@@ -141,12 +143,15 @@ public:
   // Preferences set via the preference dialog.
   Tools      tools;             //!< Toolbar composition.
   double     gamma;             //!< Gamma value of the target screen.
+  int        resolution;        //!< Forced screen resolution or zero.
   qlonglong  cacheSize;         //!< Size of the decoded page cache.
   int        pixelCacheSize;    //!< Size of the pixel cache.
   int        lensSize;          //!< Size of the magnification lens.
   int        lensPower;         //!< Power of the magnification lens.
   QString    browserProgram;    //!< Preferred web browser.
   QUrl       proxyUrl;          //!< Proxy information.
+  bool       advancedFeatures;  //!< Advanced features.
+  bool       showTextLabel;     //!< Show text label.
   Qt::KeyboardModifiers modifiersForLens;   //!< Keys for the lens. 
   Qt::KeyboardModifiers modifiersForSelect; //!< Keys for selecting.
   Qt::KeyboardModifiers modifiersForLinks;  //!< Keys for showing the links.
@@ -216,6 +221,7 @@ protected slots:
   void modeComboChanged(int);
   void zoomComboEdited();
   void setModified();
+  void forceResolutionChanged();
 private:
   QDjViewPrefsDialog();
   void connectModified(QWidget *w);
