@@ -15,7 +15,7 @@
 //C- GNU General Public License for more details.
 //C-  ------------------------------------------------------------------
 
-// $Id: qdjviewsidebar.cpp,v 1.46 2007/12/31 20:09:40 leonb Exp $
+// $Id: qdjviewsidebar.cpp,v 1.54 2008/07/31 22:14:31 leonb Exp $
 
 #if AUTOCONF
 # include "config.h"
@@ -29,6 +29,7 @@
 #include <QAction>
 #include <QActionGroup>
 #include <QApplication>
+#include <QComboBox>
 #include <QContextMenuEvent>
 #include <QCheckBox>
 #include <QDebug>
@@ -194,23 +195,29 @@ QDjViewOutline::fillItems(QTreeWidgetItem *root, miniexp_t expr)
         {
           // fill item
           const char *name = miniexp_to_str(miniexp_car(s));
-          const char *page = miniexp_to_str(miniexp_cadr(s));
+          const char *link = miniexp_to_str(miniexp_cadr(s));
           int pageno = -1;
-          if (page[0]=='#' && page[1]!='+' && page[1]!='-')
-            pageno = djview->pageNumber(QString::fromUtf8(page));
+          if (link && link[0]=='#')
+            pageno = djview->pageNumber(QString::fromUtf8(link+1));
           QTreeWidgetItem *item = new QTreeWidgetItem(root);
           if (name && name[0])
             item->setText(0, QString::fromUtf8(name));
           else if (pageno >= 0)
             item->setText(0, tr("Page %1").arg(djview->pageName(pageno)));
-          item->setData(0, Qt::UserRole, pageno);
           item->setFlags(0);
           item->setWhatsThis(0, whatsThis());
-          if (pageno >= 0)
+          if (link && link[0]=='#' && pageno>=0)
             {
-              QString name = djview->pageName(pageno);
+              item->setData(0, Qt::UserRole, pageno);
               item->setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);
-              item->setToolTip(0, tr("Go to page %1").arg(name));
+              QString pagename = djview->pageName(pageno);
+              item->setToolTip(0, tr("Go to page %1").arg(pagename));
+            } 
+          else if (link && link[0])
+            {
+              item->setData(0, Qt::UserRole, QString::fromUtf8(link));
+              item->setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);
+              item->setToolTip(0, QString::fromUtf8(link));
             }
           // recurse
           fillItems(item, miniexp_cddr(s));
@@ -251,11 +258,15 @@ void
 QDjViewOutline::searchItem(QTreeWidgetItem *item, int pageno, 
                            QTreeWidgetItem *&fi, int &fp)
 {
-  int page = item->data(0, Qt::UserRole).toInt();
-  if (page>=0 && page<=pageno && page>fp)
+  QVariant data = item->data(0, Qt::UserRole);
+  if (data.type() == QVariant::Int)
     {
-      fi = item;
-      fp = page;
+      int page = data.toInt();
+      if (page>=0 && page<=pageno && page>fp)
+        {
+          fi = item;
+          fp = page;
+        }
     }
   for (int i=0; i<item->childCount(); i++)
     searchItem(item->child(i), pageno, fi, fp);
@@ -265,9 +276,19 @@ QDjViewOutline::searchItem(QTreeWidgetItem *item, int pageno,
 void 
 QDjViewOutline::itemActivated(QTreeWidgetItem *item)
 {
-  int pageno = item->data(0, Qt::UserRole).toInt();
-  if (pageno >= 0 && pageno < djview->pageNum())
-    djview->goToPage(pageno);
+  QVariant data = item->data(0, Qt::UserRole);
+  if (data.type() == QVariant::Int)
+    {
+      int page = data.toInt();
+      if (page >= 0 && page < djview->pageNum())
+        djview->goToPage(page);
+    }
+  else if (data.type() == QVariant::String)
+    {
+      QString link = data.toString();
+      if (link.size() > 0)
+        djview->goToLink(link);
+    }
 }
 
 
@@ -475,7 +496,7 @@ QDjViewThumbnails::Model::scheduleRefresh()
 }
 
 
-void
+void 
 QDjViewThumbnails::Model::refresh()
 {
   QDjVuDocument *doc = djview->getDocument();
@@ -564,8 +585,13 @@ QDjViewThumbnails::Model::makeIcon(int pageno) const
       int w = size;
       int h = size;
       QImage img(size, size, QImage::Format_RGB32);
-      if (ddjvu_thumbnail_render(*doc, pageno, &w, &h, format, 
-                                 img.bytesPerLine(), (char*)img.bits() ) )
+      int status = ddjvu_thumbnail_status(*doc, pageno, 0);
+      if (status == DDJVU_JOB_NOTSTARTED)
+        {
+          const_cast<Model*>(this)->scheduleRefresh();
+        }
+      else if (ddjvu_thumbnail_render(*doc, pageno, &w, &h, format, 
+                                      img.bytesPerLine(), (char*)img.bits() ))
         {
           QPixmap pixmap(size,size);
           pixmap.fill();
@@ -1034,7 +1060,7 @@ QDjViewFind::Model::documentClosed(QDjVuDocument *doc)
   searchBackwards = false;
   pending = false;
   widget->eraseText();
-  widget->edit->setEnabled(false);
+  widget->combo->setEnabled(false);
   widget->label->setText(QString());
   widget->stack->setCurrentIndex(0);
 }
@@ -1048,7 +1074,7 @@ QDjViewFind::Model::documentReady(QDjVuDocument *doc)
   curHit = -1;
   if (doc)
     {
-      widget->edit->setEnabled(true);
+      widget->combo->setEnabled(true);
       connect(doc, SIGNAL(pageinfo()), this, SLOT(pageinfo()));
       connect(doc, SIGNAL(idle()), this, SLOT(pageinfo()));
       if (! find.isEmpty())
@@ -1184,7 +1210,7 @@ QDjViewFind::Model::doHighlights(int pageno)
               if (miniexp_get_rect(exp, rect))
                 djvu->addHighlight(pageno, rect.x(), rect.y(),
                                    rect.width(), rect.height(),
-                                   color );
+                                   color, true );
             }
         }
     }
@@ -1323,7 +1349,7 @@ QDjViewFind::Model::workTimeout()
               if (! somePagesWithText)
                 {
                   widget->eraseText();
-                  widget->edit->setEnabled(false);
+                  widget->combo->setEnabled(false);
                   msg = tr("<html>Document is not searchable. "
                            "No page contains information "
                            "about its textual content.</html>");
@@ -1365,7 +1391,7 @@ QDjViewFind::Model::nextHit(bool backwards)
     {
       pending = true;
       doPending();
-      if (working && pending>0 && !workTimer->isActive())
+      if (working && pending && !workTimer->isActive())
         workTimer->start(0);
     }
 }
@@ -1514,8 +1540,12 @@ QDjViewFind::QDjViewFind(QDjView *djview)
   QBoxLayout *vlayout = new QVBoxLayout(this);
   QToolBar *tools = new QToolBar(this);
   tools->addAction(eraseAction);
-  edit = new QLineEdit(tools);
-  tools->addWidget(edit);
+  combo = new QComboBox(tools);
+  combo->setEditable(true);
+  combo->setMaxCount(8);
+  combo->setInsertPolicy(QComboBox::InsertAtTop);
+  combo->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
+  tools->addWidget(combo);
   tools->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
   vlayout->addWidget(tools);
   QBoxLayout *hlayout = new QHBoxLayout;
@@ -1556,9 +1586,9 @@ QDjViewFind::QDjViewFind(QDjView *djview)
           model, SLOT(itemActivated(const QModelIndex&)));
   connect(djview->getDjVuWidget(), SIGNAL(pageChanged(int)),
           this, SLOT(pageChanged(int)));
-  connect(edit, SIGNAL(textChanged(QString)),
+  connect(combo->lineEdit(), SIGNAL(textChanged(QString)),
           model, SLOT(textChanged()));
-  connect(edit, SIGNAL(returnPressed()),
+  connect(combo->lineEdit(), SIGNAL(returnPressed()),
           this, SLOT(findAgain()));
   connect(eraseAction, SIGNAL(triggered()), 
           this, SLOT(eraseText()));
@@ -1597,15 +1627,15 @@ QDjViewFind::contextMenuEvent(QContextMenuEvent *event)
 void
 QDjViewFind::takeFocus(Qt::FocusReason reason)
 {
-  if (edit->isVisible())
-    edit->setFocus(reason);
+  if (combo->isVisible())
+    combo->setFocus(reason);
 }
 
 
 QString 
 QDjViewFind::text()
 {
-  return edit->text();
+  return combo->lineEdit()->text();
 }
 
 
@@ -1627,7 +1657,15 @@ void
 QDjViewFind::setText(QString s)
 {
   if (s != text())
-    edit->setText(s);
+    combo->lineEdit()->setText(s);
+}
+
+
+void 
+QDjViewFind::selectAll()
+{
+  combo->lineEdit()->selectAll();
+  combo->lineEdit()->setFocus();
 }
 
 
