@@ -15,7 +15,7 @@
 //C- GNU General Public License for more details.
 //C-  ------------------------------------------------------------------
 
-// $Id: qdjviewsidebar.h,v 1.13 2007/03/25 21:02:13 leonb Exp $
+// $Id: qdjviewsidebar.h,v 1.15 2008/07/31 22:14:31 leonb Exp $
 
 #ifndef QDJVIEWSIDEBAR_H
 #define QDJVIEWSIDEBAR_H
@@ -39,6 +39,7 @@
 #include "qdjview.h"
 
 class QAction;
+class QComboBox;
 class QContextMenuEvent;
 class QItemSelectionModel;
 class QLabel;
@@ -129,6 +130,7 @@ public:
   bool caseSensitive();
   bool wordOnly();
 public slots:
+  void selectAll();
   void eraseText();
   void setText(QString s);
   void setCaseSensitive(bool);
@@ -146,7 +148,7 @@ private:
   Model          *model;
   QListView      *view;
   QMenu          *menu;
-  QLineEdit      *edit;
+  QComboBox      *combo;
   QPushButton    *upButton;
   QPushButton    *downButton;
   QStackedLayout *stack;
