@@ -15,7 +15,7 @@
 //C- GNU General Public License for more details.
 //C-  ------------------------------------------------------------------
 
-// $Id: qdjvu.cpp,v 1.30 2007/11/06 13:27:57 leonb Exp $
+// $Id: qdjvu.cpp,v 1.31 2008/01/28 22:51:50 leonb Exp $
 
 #if AUTOCONF
 # include "config.h"
@@ -106,9 +106,8 @@ QDjVuContext::callback(ddjvu_context_t *, void *closure)
   if (! qcontext->flag)
     {
       qcontext->flag = true;
-      QCoreApplication *app = QCoreApplication::instance();
       QEvent *qevent = new QEvent(QEvent::User);
-      app->postEvent(qcontext, qevent);
+	  QCoreApplication::postEvent(qcontext, qevent);
     }
 }
 
