@@ -15,7 +15,7 @@
 //C- GNU General Public License for more details.
 //C-  ------------------------------------------------------------------
 
-// $Id: qdjvuwidget.h,v 1.23 2007/11/04 02:42:41 leonb Exp $
+// $Id: qdjvuwidget.h,v 1.34 2008/08/08 11:37:46 leonb Exp $
 
 #ifndef QDJVUWIDGET_H
 #define QDJVUWIDGET_H
@@ -47,6 +47,8 @@ class QDjVuWidget : public QAbstractScrollArea
              READ zoom WRITE setZoom)
   Q_PROPERTY(double gamma 
              READ gamma WRITE setGamma)
+  Q_PROPERTY(double screenDpi 
+             READ screenDpi WRITE setScreenDpi)
   Q_PROPERTY(DisplayMode displayMode 
              READ displayMode WRITE setDisplayMode)
   Q_PROPERTY(bool displayFrame 
@@ -55,14 +57,20 @@ class QDjVuWidget : public QAbstractScrollArea
              READ horizAlign WRITE setHorizAlign)
   Q_PROPERTY(Align vertAlign 
              READ vertAlign WRITE setVertAlign)
-  Q_PROPERTY(bool sideBySide 
-             READ sideBySide WRITE setSideBySide)
   Q_PROPERTY(bool continuous 
              READ continuous WRITE setContinuous)
+  Q_PROPERTY(bool sideBySide 
+             READ sideBySide WRITE setSideBySide)
+  Q_PROPERTY(bool coverPage 
+             READ coverPage WRITE setCoverPage)
+  Q_PROPERTY(bool rightToLeft 
+             READ rightToLeft WRITE setRightToLeft)
   Q_PROPERTY(QBrush borderBrush 
              READ borderBrush WRITE setBorderBrush)
   Q_PROPERTY(int borderSize 
              READ borderSize WRITE setBorderSize)
+  Q_PROPERTY(int separatorSize 
+             READ separatorSize WRITE setSeparatorSize)
   Q_PROPERTY(int pixelCacheSize 
              READ pixelCacheSize WRITE setPixelCacheSize)
   Q_PROPERTY(bool displayMapAreas 
@@ -112,7 +120,7 @@ public:
   enum Align {
     ALIGN_TOP,                  //!< Align page top sides.
     ALIGN_LEFT = ALIGN_TOP,     //!< Align page left sides.
-    ALIGN_CENTER,               //!< Center pages
+    ALIGN_CENTER,               //!< Center pages.
     ALIGN_BOTTOM,               //!< Align page bottom sides.
     ALIGN_RIGHT = ALIGN_BOTTOM, //!< Align page right sides.
   };
@@ -129,16 +137,19 @@ public:
     QPoint posPage;
     QPoint posView;
     bool   inPage;
-    bool   anchorRight;
-    bool   anchorBottom;
     bool   valid;
+    char   hAnchor;
+    char   vAnchor;
     Position();
   };
 
   struct PageInfo {
-    int width;
-    int height;
-    int dpi;
+    int   pageno;
+    int   dpi;
+    int   width;
+    int   height;
+    QRect segment;
+    QRect selected;
   };
 
   ~QDjVuWidget();
@@ -155,14 +166,18 @@ public:
   int zoom(void) const;
   int zoomFactor(void) const;
   double gamma(void) const;
+  int screenDpi(void) const;
   DisplayMode displayMode(void) const;
   bool displayFrame(void) const;
   Align horizAlign(void) const;
   Align vertAlign(void) const;
-  bool sideBySide(void) const;
   bool continuous(void) const;
+  bool sideBySide(void) const;
+  bool coverPage(void) const;
+  bool rightToLeft(void) const;
   QBrush borderBrush(void) const;
   int borderSize(void) const;
+  int separatorSize(void) const;
   QMenu* contextMenu(void) const;
   int  pixelCacheSize(void) const;
   bool displayMapAreas(void) const;
@@ -183,14 +198,18 @@ public slots:
   void setRotation(int);
   void setZoom(int);
   void setGamma(double);
+  void setScreenDpi(int);
   void setDisplayMode(DisplayMode m);
   void setDisplayFrame(bool);
   void setHorizAlign(Align);
   void setVertAlign(Align);
-  void setSideBySide(bool);
   void setContinuous(bool);
+  void setSideBySide(bool);
+  void setCoverPage(bool);
+  void setRightToLeft(bool);
   void setBorderBrush(QBrush);
   void setBorderSize(int);
+  void setSeparatorSize(int);
   void setContextMenu(QMenu*);
   void setDisplayMapAreas(bool);
   void setPixelCacheSize(int);
@@ -216,10 +235,14 @@ public:
   QString linkTarget(void);
   QString linkComment(void);
   QString getTextForRect(const QRect &rect);
+  bool    getTextForPointer(QString results[]);
   QImage  getImageForRect(const QRect &rect);
+  QRect   getSegmentForRect(const QRect &rect, int pageNo);
+
   bool pageSizeKnown(int pageno) const;
   void clearHighlights(int pageno);
-  void addHighlight(int pageno, int x, int y, int w, int h, QColor color);
+  void addHighlight(int pageno, int x, int y, int w, int h, 
+                    QColor color, bool rc=false);
   
 protected:
   virtual bool viewportEvent (QEvent *event);
@@ -232,6 +255,7 @@ protected:
   virtual void mouseReleaseEvent(QMouseEvent *event);
   virtual void mouseMoveEvent(QMouseEvent *event);
   virtual void contextMenuEvent (QContextMenuEvent *event);
+  virtual void wheelEvent(QWheelEvent *e);
   virtual void paintEvent(QPaintEvent *event);
   virtual void chooseTooltip(void);
   virtual void paintDesk(QPainter &p, const QRegion &region);
