@@ -15,7 +15,7 @@
 //C- GNU General Public License for more details.
 //C-  ------------------------------------------------------------------
 
-// $Id: tiff2pdf.h,v 1.3 2007/06/10 23:13:08 leonb Exp $
+// $Id: tiff2pdf.h,v 1.4 2008/08/05 06:03:37 leonb Exp $
 
 #ifndef TIFF2PDF_H
 # define TIFF2PDF_H
@@ -25,7 +25,9 @@
 # if HAVE_TIFF
 #  include <stdio.h>
 #  include <stdlib.h>
-#  include "tiffio.h"
+#  include <tiff.h>
+#  include <tiffio.h>
+#  include <tiffconf.h>
 #  ifdef TIFFLIB_VERSION
 #   if TIFFLIB_VERSION > 20041104
 #    define HAVE_TIFF2PDF 1
